@@ -1401,38 +1401,42 @@ impl DrawingCtx {
                 for &target in &paint_order.targets {
                     match target {
                         PaintTarget::Fill => {
-                            self.cr.move_to(span.x, span.y);
-
-                            let matrix = self.cr.matrix();
-                            if let Some(rot) = rotation {
-                                self.cr.rotate(rot);
-                            }
-
                             let fill_paint =
                                 span.fill_paint.to_user_space(&bbox, view_params, values);
                             let had_paint_server =
                                 self.set_paint_source(&fill_paint, acquired_nodes)?;
+
                             if had_paint_server {
+                                self.cr.move_to(span.x, span.y);
+
+                                let matrix = self.cr.matrix();
+                                if let Some(rot) = rotation {
+                                    self.cr.rotate(rot);
+                                }
+
                                 pangocairo::functions::update_layout(&self.cr, &span.layout);
                                 pangocairo::functions::show_layout(&self.cr, &span.layout);
-                            }
 
-                            self.cr.set_matrix(matrix);
+                                self.cr.set_matrix(matrix);
+
+                                self.cr.new_path();
+                            }
                         }
 
                         PaintTarget::Stroke => {
-                            self.cr.move_to(span.x, span.y);
-
-                            let matrix = self.cr.matrix();
-                            if let Some(rot) = rotation {
-                                self.cr.rotate(rot);
-                            }
-
                             let stroke_paint =
                                 span.stroke_paint.to_user_space(&bbox, view_params, values);
                             let had_paint_server =
                                 self.set_paint_source(&stroke_paint, acquired_nodes)?;
+
                             if had_paint_server {
+                                self.cr.move_to(span.x, span.y);
+
+                                let matrix = self.cr.matrix();
+                                if let Some(rot) = rotation {
+                                    self.cr.rotate(rot);
+                                }
+
                                 pangocairo::functions::update_layout(&self.cr, &span.layout);
                                 pangocairo::functions::layout_path(&self.cr, &span.layout);
 
@@ -1443,9 +1447,11 @@ impl DrawingCtx {
                                     .with_ink_rect(r);
                                 bbox.insert(&ib);
                                 self.cr.stroke()?;
-                            }
 
-                            self.cr.set_matrix(matrix);
+                                self.cr.set_matrix(matrix);
+
+                                self.cr.new_path();
+                            }
                         }
 
                         PaintTarget::Markers => {}
@@ -1720,6 +1726,48 @@ impl DrawingCtx {
             res
         }
     }
+
+    /// Create a Pango context based on the cr and `testing` flag from the DrawingCtx.
+    pub fn create_pango_context(&self) -> pango::Context {
+        let cr = self.cr.clone();
+
+        let mut options = cairo::FontOptions::new().unwrap();
+        if self.testing {
+            options.set_antialias(cairo::Antialias::Gray);
+        }
+
+        options.set_hint_style(cairo::HintStyle::None);
+        options.set_hint_metrics(cairo::HintMetrics::Off);
+
+        cr.set_font_options(&options);
+
+        let font_map = pangocairo::FontMap::default().unwrap();
+        let context = font_map.create_context().unwrap();
+
+        context.set_round_glyph_positions(false);
+
+        pangocairo::functions::update_context(&cr, &context);
+
+        // Pango says this about pango_cairo_context_set_resolution():
+        //
+        //     Sets the resolution for the context. This is a scale factor between
+        //     points specified in a #PangoFontDescription and Cairo units. The
+        //     default value is 96, meaning that a 10 point font will be 13
+        //     units high. (10 * 96. / 72. = 13.3).
+        //
+        // I.e. Pango font sizes in a PangoFontDescription are in *points*, not pixels.
+        // However, we are normalizing everything to userspace units, which amount to
+        // pixels.  So, we will use 72.0 here to make Pango not apply any further scaling
+        // to the size values we give it.
+        //
+        // An alternative would be to divide our font sizes by (dpi_y / 72) to effectively
+        // cancel out Pango's scaling, but it's probably better to deal with Pango-isms
+        // right here, instead of spreading them out through our Length normalization
+        // code.
+        pangocairo::functions::context_set_resolution(&context, 72.0);
+
+        context
+    }
 }
 
 // https://www.w3.org/TR/css-masking-1/#ClipPathElement
@@ -2041,49 +2089,6 @@ impl From<TextRendering> for cairo::Antialias {
     }
 }
 
-impl From<&DrawingCtx> for pango::Context {
-    fn from(draw_ctx: &DrawingCtx) -> pango::Context {
-        let cr = draw_ctx.cr.clone();
-
-        let mut options = cairo::FontOptions::new().unwrap();
-        if draw_ctx.testing {
-            options.set_antialias(cairo::Antialias::Gray);
-        }
-
-        options.set_hint_style(cairo::HintStyle::None);
-        options.set_hint_metrics(cairo::HintMetrics::Off);
-
-        cr.set_font_options(&options);
-
-        let font_map = pangocairo::FontMap::default().unwrap();
-        let context = font_map.create_context().unwrap();
-
-        context.set_round_glyph_positions(false);
-
-        pangocairo::functions::update_context(&cr, &context);
-
-        // Pango says this about pango_cairo_context_set_resolution():
-        //
-        //     Sets the resolution for the context. This is a scale factor between
-        //     points specified in a #PangoFontDescription and Cairo units. The
-        //     default value is 96, meaning that a 10 point font will be 13
-        //     units high. (10 * 96. / 72. = 13.3).
-        //
-        // I.e. Pango font sizes in a PangoFontDescription are in *points*, not pixels.
-        // However, we are normalizing everything to userspace units, which amount to
-        // pixels.  So, we will use 72.0 here to make Pango not apply any further scaling
-        // to the size values we give it.
-        //
-        // An alternative would be to divide our font sizes by (dpi_y / 72) to effectively
-        // cancel out Pango's scaling, but it's probably better to deal with Pango-isms
-        // right here, instead of spreading them out through our Length normalization
-        // code.
-        pangocairo::functions::context_set_resolution(&context, 72.0);
-
-        context
-    }
-}
-
 impl From<cairo::Matrix> for Transform {
     #[inline]
     fn from(m: cairo::Matrix) -> Self {
