@@ -6,7 +6,7 @@ use std::rc::Rc;
 
 use crate::bbox::BoundingBox;
 use crate::document::{AcquiredNodes, NodeId};
-use crate::drawing_ctx::DrawingCtx;
+use crate::drawing_ctx::{DrawingCtx, ViewParams};
 use crate::element::{Draw, Element, ElementResult, SetAttributes};
 use crate::error::*;
 use crate::layout::{self, FontProperties, StackingContext, Stroke};
@@ -74,10 +74,7 @@ struct MeasuredSpan {
 struct PositionedSpan {
     layout: pango::Layout,
     values: Rc<ComputedValues>,
-    _position: (f64, f64),
     rendered_position: (f64, f64),
-    next_span_x: f64,
-    next_span_y: f64,
 }
 
 impl Chunk {
@@ -92,11 +89,15 @@ impl Chunk {
 }
 
 impl MeasuredChunk {
-    fn from_chunk(chunk: &Chunk, draw_ctx: &DrawingCtx) -> MeasuredChunk {
+    fn from_chunk(
+        chunk: &Chunk,
+        text_writing_mode: WritingMode,
+        draw_ctx: &DrawingCtx,
+    ) -> MeasuredChunk {
         let measured_spans: Vec<MeasuredSpan> = chunk
             .spans
             .iter()
-            .map(|span| MeasuredSpan::from_span(span, draw_ctx))
+            .map(|span| MeasuredSpan::from_span(span, text_writing_mode, draw_ctx))
             .collect();
 
         let advance = measured_spans.iter().fold((0.0, 0.0), |acc, measured| {
@@ -116,32 +117,66 @@ impl MeasuredChunk {
 impl PositionedChunk {
     fn from_measured(
         measured: &MeasuredChunk,
-        draw_ctx: &DrawingCtx,
+        view_params: &ViewParams,
+        text_writing_mode: WritingMode,
         x: f64,
         y: f64,
     ) -> PositionedChunk {
         let mut positioned = Vec::new();
 
-        // Adjust the specified coordinates with the text_anchor
-
-        let adjusted_advance = text_anchor_advance(
+        // measured.advance is the size of the chunk.  Compute the offsets needed to align
+        // it per the text-anchor property (start, middle, end):
+        let anchor_offset = text_anchor_offset(
             measured.values.text_anchor(),
-            measured.values.writing_mode(),
+            measured.values.direction(),
+            text_writing_mode,
             measured.advance,
         );
 
-        let mut x = x + adjusted_advance.0;
-        let mut y = y + adjusted_advance.1;
+        let mut x = x + anchor_offset.0;
+        let mut y = y + anchor_offset.1;
 
         // Position each span
 
-        for measured_span in &measured.spans {
-            let positioned_span = PositionedSpan::from_measured(measured_span, draw_ctx, x, y);
+        for mspan in &measured.spans {
+            let params = NormalizeParams::new(&mspan.values, view_params);
 
-            x = positioned_span.next_span_x;
-            y = positioned_span.next_span_y;
+            let layout = mspan.layout.clone();
+            let values = mspan.values.clone();
+
+            let baseline = f64::from(layout.baseline()) / f64::from(pango::SCALE);
+            let baseline_shift = values.baseline_shift().0.to_user(&params);
+            let baseline_offset = baseline + baseline_shift;
+
+            let dx = mspan.dx;
+            let dy = mspan.dy;
+
+            let start_pos = match values.direction() {
+                Direction::Ltr => (x, y),
+                Direction::Rtl => (x - mspan.advance.0, y),
+            };
+
+            let span_advance = match values.direction() {
+                Direction::Ltr => (mspan.advance.0, mspan.advance.1),
+                Direction::Rtl => (-mspan.advance.0, mspan.advance.1),
+            };
+
+            let rendered_position = if text_writing_mode.is_horizontal() {
+                (start_pos.0 + dx, start_pos.1 - baseline_offset + dy)
+            } else {
+                (start_pos.0 + baseline_offset + dx, start_pos.1 + dy)
+            };
+
+            let positioned_span = PositionedSpan {
+                layout,
+                values,
+                rendered_position,
+            };
 
             positioned.push(positioned_span);
+
+            x = x + span_advance.0 + dx;
+            y = y + span_advance.1 + dy;
         }
 
         PositionedChunk {
@@ -152,22 +187,33 @@ impl PositionedChunk {
     }
 }
 
-fn text_anchor_advance(
+/// Computes the (x, y) offsets to be applied to spans after applying the text-anchor property (start, middle, end).
+#[rustfmt::skip]
+fn text_anchor_offset(
     anchor: TextAnchor,
-    writing_mode: WritingMode,
-    advance: (f64, f64),
+    direction: Direction,
+    text_writing_mode: WritingMode,
+    chunk_size: (f64, f64),
 ) -> (f64, f64) {
-    if writing_mode.is_vertical() {
-        match anchor {
-            TextAnchor::Start => (0.0, 0.0),
-            TextAnchor::Middle => (0.0, -advance.1 / 2.0),
-            TextAnchor::End => (0.0, -advance.1),
+    let (w, h) = chunk_size;
+
+    if text_writing_mode.is_horizontal() {
+        match (anchor, direction) {
+            (TextAnchor::Start,  Direction::Ltr) => (0.0, 0.0),
+            (TextAnchor::Start,  Direction::Rtl) => (0.0, 0.0),
+
+            (TextAnchor::Middle, Direction::Ltr) => (-w / 2.0, 0.0),
+            (TextAnchor::Middle, Direction::Rtl) => (w / 2.0, 0.0),
+
+            (TextAnchor::End,    Direction::Ltr) => (-w, 0.0),
+            (TextAnchor::End,    Direction::Rtl) => (w, 0.0),
         }
     } else {
+        // FIXME: we don't deal with text direction for vertical text yet.
         match anchor {
             TextAnchor::Start => (0.0, 0.0),
-            TextAnchor::Middle => (-advance.0 / 2.0, 0.0),
-            TextAnchor::End => (-advance.0, 0.0),
+            TextAnchor::Middle => (0.0, -h / 2.0),
+            TextAnchor::End => (0.0, -h),
         }
     }
 }
@@ -185,23 +231,31 @@ impl Span {
 }
 
 impl MeasuredSpan {
-    fn from_span(span: &Span, draw_ctx: &DrawingCtx) -> MeasuredSpan {
+    fn from_span(
+        span: &Span,
+        text_writing_mode: WritingMode,
+        draw_ctx: &DrawingCtx,
+    ) -> MeasuredSpan {
         let values = span.values.clone();
 
         let view_params = draw_ctx.get_view_params();
         let params = NormalizeParams::new(&values, &view_params);
 
-        let properties = FontProperties::new(&values, &params);
+        let properties = FontProperties::new(&values, text_writing_mode, &params);
         let layout = create_pango_layout(draw_ctx, &properties, &span.text);
         let (w, h) = layout.size();
 
         let w = f64::from(w) / f64::from(pango::SCALE);
         let h = f64::from(h) / f64::from(pango::SCALE);
 
-        let advance = if values.writing_mode().is_vertical() {
-            (0.0, w)
-        } else {
+        // This is the logical size of the layout, regardless of text direction, so it's always positive.
+        assert!(w >= 0.0);
+        assert!(h >= 0.0);
+
+        let advance = if text_writing_mode.is_horizontal() {
             (w, 0.0)
+        } else {
+            (0.0, w)
         };
 
         MeasuredSpan {
@@ -216,41 +270,6 @@ impl MeasuredSpan {
 }
 
 impl PositionedSpan {
-    fn from_measured(
-        measured: &MeasuredSpan,
-        draw_ctx: &DrawingCtx,
-        x: f64,
-        y: f64,
-    ) -> PositionedSpan {
-        let layout = measured.layout.clone();
-        let values = measured.values.clone();
-
-        let view_params = draw_ctx.get_view_params();
-        let params = NormalizeParams::new(&values, &view_params);
-
-        let baseline = f64::from(layout.baseline()) / f64::from(pango::SCALE);
-        let baseline_shift = values.baseline_shift().0.to_user(&params);
-        let offset = baseline + baseline_shift;
-
-        let dx = measured.dx;
-        let dy = measured.dy;
-
-        let (render_x, render_y) = if values.writing_mode().is_vertical() {
-            (x + offset + dx, y + dy)
-        } else {
-            (x + dx, y - offset + dy)
-        };
-
-        PositionedSpan {
-            layout: measured.layout.clone(),
-            values,
-            _position: (x, y),
-            rendered_position: (render_x, render_y),
-            next_span_x: x + measured.advance.0 + dx,
-            next_span_y: y + measured.advance.1 + dy,
-        }
-    }
-
     fn draw(
         &self,
         acquired_nodes: &mut AcquiredNodes<'_>,
@@ -302,10 +321,6 @@ impl PositionedSpan {
 /// Walks the children of a `<text>`, `<tspan>`, or `<tref>` element
 /// and appends chunks/spans from them into the specified `chunks`
 /// array.
-///
-/// `x` and `y` are the absolute position for the first chunk.  If the
-/// first child is a `<tspan>` with a specified absolute position, it
-/// will be used instead of the given arguments.
 fn children_to_chunks(
     chunks: &mut Vec<Chunk>,
     node: &Node,
@@ -535,6 +550,8 @@ impl Draw for Text {
 
         let stacking_ctx = StackingContext::new(acquired_nodes, &elt, values.transform(), values);
 
+        let text_writing_mode = values.writing_mode();
+
         draw_ctx.with_discrete_layer(
             &stacking_ctx,
             acquired_nodes,
@@ -549,7 +566,7 @@ impl Draw for Text {
 
                 let mut measured_chunks = Vec::new();
                 for chunk in &chunks {
-                    measured_chunks.push(MeasuredChunk::from_chunk(chunk, dc));
+                    measured_chunks.push(MeasuredChunk::from_chunk(chunk, text_writing_mode, dc));
                 }
 
                 let mut positioned_chunks = Vec::new();
@@ -557,7 +574,13 @@ impl Draw for Text {
                     let chunk_x = chunk.x.unwrap_or(x);
                     let chunk_y = chunk.y.unwrap_or(y);
 
-                    let positioned = PositionedChunk::from_measured(chunk, dc, chunk_x, chunk_y);
+                    let positioned = PositionedChunk::from_measured(
+                        chunk,
+                        &view_params,
+                        text_writing_mode,
+                        chunk_x,
+                        chunk_y,
+                    );
 
                     x = positioned.next_chunk_x;
                     y = positioned.next_chunk_y;
@@ -789,28 +812,27 @@ impl From<Direction> for pango::Alignment {
 
 impl From<WritingMode> for pango::Direction {
     fn from(m: WritingMode) -> pango::Direction {
+        use WritingMode::*;
         match m {
-            WritingMode::LrTb | WritingMode::Lr | WritingMode::Tb | WritingMode::TbRl => {
-                pango::Direction::Ltr
-            }
-            WritingMode::RlTb | WritingMode::Rl => pango::Direction::Rtl,
+            HorizontalTb | VerticalRl | VerticalLr | LrTb | Lr | Tb | TbRl => pango::Direction::Ltr,
+            RlTb | Rl => pango::Direction::Rtl,
         }
     }
 }
 
 impl From<WritingMode> for pango::Gravity {
     fn from(m: WritingMode) -> pango::Gravity {
+        use WritingMode::*;
         match m {
-            WritingMode::Tb | WritingMode::TbRl => pango::Gravity::East,
-            WritingMode::LrTb | WritingMode::Lr | WritingMode::RlTb | WritingMode::Rl => {
-                pango::Gravity::South
-            }
+            HorizontalTb | LrTb | Lr | RlTb | Rl => pango::Gravity::South,
+            VerticalRl | Tb | TbRl => pango::Gravity::East,
+            VerticalLr => pango::Gravity::West,
         }
     }
 }
 
 fn create_pango_layout(draw_ctx: &DrawingCtx, props: &FontProperties, text: &str) -> pango::Layout {
-    let pango_context = pango::Context::from(draw_ctx);
+    let pango_context = draw_ctx.create_pango_context();
 
     if let XmlLang(Some(ref lang)) = props.xml_lang {
         pango_context.set_language(&pango::Language::from_string(lang));
@@ -891,7 +913,7 @@ fn create_pango_layout(draw_ctx: &DrawingCtx, props: &FontProperties, text: &str
 
 #[cfg(test)]
 mod tests {
-    use super::Chars;
+    use super::*;
 
     #[test]
     fn chars_default() {
@@ -907,4 +929,71 @@ mod tests {
         assert_eq!(c.get_string(), example);
         assert!(c.space_normalized.borrow().is_none());
     }
+
+    // This is called _horizontal because the property value in "CSS Writing Modes 3"
+    // is `horizontal-tb`.  Eventually we will support that and this will make more sense.
+    #[test]
+    fn adjusted_advance_horizontal_ltr() {
+        use Direction::*;
+        use TextAnchor::*;
+
+        assert_eq!(
+            text_anchor_offset(Start, Ltr, WritingMode::Lr, (2.0, 4.0)),
+            (0.0, 0.0)
+        );
+
+        assert_eq!(
+            text_anchor_offset(Middle, Ltr, WritingMode::Lr, (2.0, 4.0)),
+            (-1.0, 0.0)
+        );
+
+        assert_eq!(
+            text_anchor_offset(End, Ltr, WritingMode::Lr, (2.0, 4.0)),
+            (-2.0, 0.0)
+        );
+    }
+
+    #[test]
+    fn adjusted_advance_horizontal_rtl() {
+        use Direction::*;
+        use TextAnchor::*;
+
+        assert_eq!(
+            text_anchor_offset(Start, Rtl, WritingMode::Rl, (2.0, 4.0)),
+            (0.0, 0.0)
+        );
+        assert_eq!(
+            text_anchor_offset(Middle, Rtl, WritingMode::Rl, (2.0, 4.0)),
+            (1.0, 0.0)
+        );
+        assert_eq!(
+            text_anchor_offset(TextAnchor::End, Direction::Rtl, WritingMode::Rl, (2.0, 4.0)),
+            (2.0, 0.0)
+        );
+    }
+
+    // This is called _vertical because "CSS Writing Modes 3" has both `vertical-rl` (East
+    // Asia), and `vertical-lr` (Manchu, Mongolian), but librsvg does not support block
+    // flow direction properly yet.  Eventually we will support that and this will make
+    // more sense.
+    #[test]
+    fn adjusted_advance_vertical() {
+        use Direction::*;
+        use TextAnchor::*;
+
+        assert_eq!(
+            text_anchor_offset(Start, Ltr, WritingMode::Tb, (2.0, 4.0)),
+            (0.0, 0.0)
+        );
+
+        assert_eq!(
+            text_anchor_offset(Middle, Ltr, WritingMode::Tb, (2.0, 4.0)),
+            (0.0, -2.0)
+        );
+
+        assert_eq!(
+            text_anchor_offset(End, Ltr, WritingMode::Tb, (2.0, 4.0)),
+            (0.0, -4.0)
+        );
+    }
 }
